/*
 * VSSClient.c
 * 
 * Example UDP/IP client that sends control messages to VSS.
 * This just picks random numbers and sends them along to the VSS server
 * once per second.
 *
 * Typically, the client and server and two different machines.  It doesn't
 * matter with UDP whether the client or server is started first.  Assume
 * that the server is named "yoyo" and the client is "yaya", then you would
 * want to start the server with:
 *
 *		VSS -u yaya -s 5013
 * 
 * and the client with
 *
 *		VSSClient yoyo 5013
 *
 * 5013 is a big number I picked out of the air.  You can use anything probably.
 *
 * You will not *see* yoyo's VSS interface changing, but you should hear it.
 * (Make sure you have distance, doppler and convolution turned on in the lower
 * right check box in the control panel).
 *
 * The VSS udp socket expects three floats per message.  They are in the order:
 * azimuth, elevation, distance.  Azimuth and elevation are measured in degrees.
 * For azimuth, straight ahead is zero, right is 90, behind is 180, left is 270.
 * Elevation should be between -40 and 90.  Distance is in meters.  VSS does not
 * allow distances less than 0.1 meters (otherwise it gets VERY loud!). 
 *
 * For checking and debugging, you may give a "-d" flag to VSS (along with the
 * -u and -s) to print out the values coming in over the UDP socket.
 *
 *
 * Author: Rick Bidlack, Oct 23, 1994
 * Copyright (c) 1994 KaV Software 
 */
 
#include <stdio.h>
#include <malloc.h>
#include <bstring.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <sys/prctl.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>

#define RAND_MAX	2147483648


extern int errno;
struct sockaddr_in sockMe;
struct sockaddr_in sockOther;

int commSetUp(struct sockaddr_in *so_me, struct sockaddr_in *so_other, 
	char *host, short port)
{
	int fd;
	struct hostent *addr;
	unsigned long addr_tmp;

	bzero ((char *)so_me, sizeof(struct sockaddr_in));
	so_me->sin_port =  htons(port);

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		return -1;
	}
	fprintf(stderr, "client socket OK\n");

	if (bind(fd, so_me, sizeof(struct sockaddr_in)) < 0) {
		perror("bind");
		return -1;
	}
	fprintf(stderr, "client bind OK\n");


	bzero((char *)so_other, sizeof(struct sockaddr_in));
	if (atoi(host) > 0 )  { 		/* Numeric */
		so_other->sin_addr.s_addr = inet_addr(host);
	} else {
		if ((addr = gethostbyname(host)) == NULL) {
			fprintf(stderr, "Error getting remote host name\n");
			return -1;
		}
 		bcopy(addr->h_addr,(char*)&addr_tmp, addr->h_length);
    	so_other->sin_addr.s_addr = addr_tmp;
	}
	so_other->sin_family = AF_INET;
	so_other->sin_port = htons(port);

	return fd;
}



main(int argc, char **argv)
{
	extern char *optarg;
	extern int optind;
	int i, c, cnt;
	float buf[3];
	char *host = NULL;
	int ctlFD;
	short ctlPort;	

	if (argc != 3) {
		fprintf(stderr, "%s%s%s%s",
			"Usage: VSSClient <hostname> <socket>\n", 
			"\t<hostname> is the name of the server running VSS\n",
			"\t<socket> is an arbitrary large number, typically 5000,\n",
			"\tand must match the socket id used on the VSS server.\n"
		);
		exit (1);
	}

	host = argv[1];
	ctlPort = atoi(argv[2]);

	if ((ctlFD = commSetUp(&sockMe, &sockOther, host, ctlPort)) == -1) {
		fprintf(stderr, "Can't establish udp connection!");
		exit (1);
	}

	srandom(time(0));

	for (i=0; i<60; i++) {
		/* azimuth */
		buf[0] = (random() / (float)RAND_MAX) * 360.0;
		/* elevation */
		buf[1] = (random() / (float)RAND_MAX) * 130.0 - 40.0;
		/* distance */
		buf[2] = (random() / (float)RAND_MAX) * 20.0;
		
		fprintf(stderr, "Send: %f %f %f\n", buf[0], buf[1], buf[2]);

		sendto(ctlFD, (char *)buf, 3*sizeof(float), 0, 
			&sockOther, sizeof(struct sockaddr_in));

		sleep(1);
	}

	close (ctlFD);
}


